#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2025 Canonical, Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest

from apparmor.common import AppArmorException, AppArmorBug, combine_profname
from apparmor.profile_storage import ProfileStorage
from apparmor.profile_list import ProfileList
from apparmor.rule.file import FileRule
from apparmor.rule.variable import VariableRule
from apparmor.rule.boolean import BooleanRule
from apparmor.rule.conditional import ConditionalBlock, AppArmorAst, Term, CompareCondition, ConditionalRule, BooleanCondition
from common_test import AATest, setup_all_loops


class TestConditional(AATest):
    """ Base class to test conditionals, profile_data does not contain any rules """
    filename = 'somefile'
    condition_contents = '\n'

    tests = (
        # ConditionalBlock                                                        clean rule
        (['if $FOO {', '} else if $BAR {', '} else {'],                           'if $FOO {' + condition_contents + '} else if $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not $FOO {', '} else if not $BAR {', '} else {'],                   'if not $FOO {' + condition_contents + '} else if not $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if defined $FOO {', '} else if not defined $BAR {', '} else {'],       'if defined $FOO {' + condition_contents + '} else if not defined $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {', '} else if defined @{VAR1} {', '} else {'], 'if not defined @{VAR2} {' + condition_contents + '} else if defined @{VAR1} {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {', '} else {'],                                'if not defined @{VAR2} {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {'],                                            'if not defined @{VAR2} {' + condition_contents + '}'),
    )

    @classmethod
    def setUpClass(cls):
        super(TestConditional, cls).setUpClass()
        cls.active_profiles = ProfileList()
        cls.profile_data = {}
        cls.condition_contents = '\n'
        (profile, hat, prof_storage) = ProfileStorage.parse('profile foo {', cls.filename, 0, False, False)
        cls.profile_data[profile] = prof_storage
        cls.active_profiles.add_rule(cls.filename, 'variable', VariableRule.create_instance('@{VAR1} = "test"'))
        cls.active_profiles.add_rule(cls.filename, 'variable', VariableRule.create_instance('@{VAR2} = "test1 test2"'))
        cls.active_profiles.add_rule(cls.filename, 'variable', VariableRule.create_instance('@{VAR3} = 10'))
        cls.active_profiles.add_rule(cls.filename, 'boolean', BooleanRule.create_instance('$FOO=true'))
        cls.active_profiles.add_rule(cls.filename, 'boolean', BooleanRule.create_instance('$BAR=false'))

    def _run_test(self, params, expected):
        conditional_block = None
        for condition in params:
            if condition == params[0]:
                conditional_block = ConditionalBlock(condition, self.active_profiles.files[self.filename])
                conditional_block.store_profile_data(self.profile_data)
            else:
                conditional_block.add_conditional(condition, self.active_profiles.files[self.filename])
                conditional_block.store_profile_data(self.profile_data)

        self.assertEqual(conditional_block.get_clean(), expected)


class TestConditionalComplex(TestConditional):
    """ Class to test complex conditionals (and, ors, comparisons and boolean operations), profile_data contains one file rule """
    # directly related to how setUpClass sets up profile_data
    condition_contents = '\n  /bin/false rix,\n\n'

    tests = (
        # ConditionalBlock                                                            clean rule
        (['if $FOO and ($BAR) {', '} else if $BAR {', '} else {'],                    'if $FOO and $BAR {' + condition_contents + '} else if $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if $FOO and ($BAR or defined @{VAR1}) {', '} else if $BAR {', '} else {'], 'if $FOO and ($BAR or defined @{VAR1}) {' + condition_contents + '} else if $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if $FOO or ($BAR or defined @{VAR1}) {', '} else if $BAR {', '} else {'],  'if $FOO or $BAR or defined @{VAR1} {' + condition_contents + '} else if $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if $FOO or ($BAR and defined @{VAR1}) {', '} else if $BAR {', '} else {'], 'if $FOO or $BAR and defined @{VAR1} {' + condition_contents + '} else if $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not $FOO {', '} else if not not $BAR {', '} else {'],                   'if not $FOO {' + condition_contents + '} else if $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if ((test in @{VAR1})) and defined $FOO {'],                               'if test in @{VAR1} and defined $FOO {' + condition_contents + '}'),
        (['if test3 in @{VAR2} {'],                                                   'if test3 in @{VAR2} {' + condition_contents + '}'),
        (['if 9 < @{VAR3} {'],                                                        'if 9 < @{VAR3} {' + condition_contents + '}'),
        (['if test in "test2 test" {'],                                               'if test in "test2 test" {' + condition_contents + '}'),
    )

    @classmethod
    def setUpClass(cls):
        super(TestConditionalComplex, cls).setUpClass()
        (profile, hat, prof_storage) = ProfileStorage.parse('profile foo {', cls.filename, 0, False, False)
        cls.profile_data[profile] = prof_storage
        cls.profile_data[profile]['file'].add(FileRule.create_instance('/bin/false rix,'))


class TestConditionalSimple(TestConditional):
    """ Class to test simple conditionals (boolean and defined operations), profile_data contains one file rule """
    # directly related to how setUpClass sets up profile_data
    condition_contents = '\n  /bin/false rix,\n\n'

    tests = (
        # ConditionalBlock                                                        clean rule
        (['if $FOO {', '} else if $BAR {', '} else {'],                           'if $FOO {' + condition_contents + '} else if $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not $FOO {', '} else if not $BAR {', '} else {'],                   'if not $FOO {' + condition_contents + '} else if not $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if defined $FOO {', '} else if not defined ${BAR} {', '} else {'],     'if defined $FOO {' + condition_contents + '} else if not defined ${BAR} {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {', '} else if defined @{VAR1} {', '} else {'], 'if not defined @{VAR2} {' + condition_contents + '} else if defined @{VAR1} {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {', '} else {'],                                'if not defined @{VAR2} {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {'],                                            'if not defined @{VAR2} {' + condition_contents + '}'),
    )

    @classmethod
    def setUpClass(cls):
        super(TestConditionalSimple, cls).setUpClass()
        (profile, hat, prof_storage) = ProfileStorage.parse('profile foo {', cls.filename, 0, False, False)
        cls.profile_data[profile] = prof_storage
        cls.profile_data[profile]['file'].add(FileRule.create_instance('/bin/false rix,'))


class TestConditionalSubProfile(TestConditional):
    """ Class to test simple conditionals, profile_data contains hat definition """
    # directly related to how setUpClass sets up profile_data
    condition_contents = '\n  /** w,\n  /bin/false rix,\n\n  ^bar {\n    /bin/true rix,\n\n  }\n'

    tests = (
        # ConditionalBlock                                                        clean rule
        (['if $FOO {', '} else if $BAR {', '} else {'],                           'if $FOO {' + condition_contents + '} else if $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not $FOO {', '} else if not $BAR {', '} else {'],                   'if not $FOO {' + condition_contents + '} else if not $BAR {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if defined $FOO {', '} else if not defined ${BAR} {', '} else {'],     'if defined $FOO {' + condition_contents + '} else if not defined ${BAR} {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {', '} else if defined @{VAR1} {', '} else {'], 'if not defined @{VAR2} {' + condition_contents + '} else if defined @{VAR1} {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {', '} else {'],                                'if not defined @{VAR2} {' + condition_contents + '} else {' + condition_contents + '}'),
        (['if not defined @{VAR2} {'],                                            'if not defined @{VAR2} {' + condition_contents + '}'),
    )

    @classmethod
    def setUpClass(cls):
        super(TestConditionalSubProfile, cls).setUpClass()
        (profile, hat, prof_storage) = ProfileStorage.parse('profile foo {', cls.filename, 0, False, False)
        cls.profile_data[profile] = prof_storage
        cls.profile_data[profile]['file'].add(FileRule.create_instance('/bin/false rix,'))
        cls.profile_data[profile]['file'].add(FileRule.create_instance('/** w,'))
        (profile, hat, prof_storage) = ProfileStorage.parse('^bar {', cls.filename, 0, profile, None)
        profname = combine_profname((profile, hat))
        cls.profile_data[profname] = prof_storage
        cls.profile_data[profname]['in_cond'] = True
        cls.profile_data[profname]['file'].add(FileRule.create_instance('/bin/true rix,'))


class TestConditionalLogprof(TestConditional):
    """ Class to test output of Logprof """
    # directly related to how setUpClass sets up profile_data
    condition_contents = '\n  /** w,\n  /bin/false rix,\n\n  ^bar {\n    /bin/true rix,\n\n  }\n'

    tests = (
        (['if $FOO {', '} else if $BAR {', '} else {'],
         ['ConditionalBlock', [['Conditional', 'if $FOO {' + condition_contents + '}'],
                               ['Conditional', 'else if $BAR {' + condition_contents + '}'],
                               ['Conditional', 'else {' + condition_contents + '}']]]),
        (['if not $FOO {', '} else if not $BAR {', '} else {'],
         ['ConditionalBlock', [['Conditional', 'if not $FOO {' + condition_contents + '}'],
                               ['Conditional', 'else if not $BAR {' + condition_contents + '}'],
                               ['Conditional', 'else {' + condition_contents + '}']]]),
        (['if defined $FOO {', '} else if not defined $BAR {', '} else {'],
         ['ConditionalBlock', [['Conditional', 'if defined $FOO {' + condition_contents + '}'],
                               ['Conditional', 'else if not defined $BAR {' + condition_contents + '}'],
                               ['Conditional', 'else {' + condition_contents + '}']]]),
        (['if not defined @{VAR2} {', '} else if defined @{VAR1} {', '} else {'],
         ['ConditionalBlock', [['Conditional', 'if not defined @{VAR2} {' + condition_contents + '}'],
                               ['Conditional', 'else if defined @{VAR1} {' + condition_contents + '}'],
                               ['Conditional', 'else {' + condition_contents + '}']]]),
        (['if not defined @{VAR2} {', '} else {'],
         ['ConditionalBlock', [['Conditional', 'if not defined @{VAR2} {' + condition_contents + '}'],
                               ['Conditional', 'else {' + condition_contents + '}']]]),
        (['if not defined @{VAR2} {'],
         ['ConditionalBlock', [['Conditional', 'if not defined @{VAR2} {' + condition_contents + '}']]]),
    )

    @classmethod
    def setUpClass(cls):
        super(TestConditionalLogprof, cls).setUpClass()
        (profile, hat, prof_storage) = ProfileStorage.parse('profile foo {', cls.filename, 0, False, False)
        cls.profile_data[profile] = prof_storage
        cls.profile_data[profile]['file'].add(FileRule.create_instance('/bin/false rix,'))
        cls.profile_data[profile]['file'].add(FileRule.create_instance('/** w,'))
        (profile, hat, prof_storage) = ProfileStorage.parse('^bar {', cls.filename, 0, profile, None)
        profname = combine_profname((profile, hat))
        cls.profile_data[profname] = prof_storage
        cls.profile_data[profname]['in_cond'] = True
        cls.profile_data[profname]['file'].add(FileRule.create_instance('/bin/true rix,'))

    def _run_test(self, params, expected):
        conditional_block = None
        for condition in params:
            if condition == params[0]:
                conditional_block = ConditionalBlock(condition, self.active_profiles.files[self.filename])
                conditional_block.store_profile_data(self.profile_data)
            else:
                conditional_block.add_conditional(condition, self.active_profiles.files[self.filename])
                conditional_block.store_profile_data(self.profile_data)

        self.assertEqual(conditional_block.logprof_header(), expected)


class TestConditionalEquality(AATest):
    """ Class to test equality tests """
    filename = 'somefile'
    profile_data = {}

    tests = (
        # should always be different
        # first condition                                   second condition
        ((['if not $FOO {'], profile_data),                 (['if $FOO {'], profile_data)),
        ((['if defined $FOO {'], profile_data),             (['if $FOO {'], profile_data)),
        ((['if $BAR {'], profile_data),                     (['if $FOO {'], profile_data)),
        ((['if $FOO {'], None),                             (['if $FOO {'], profile_data)),
        ((['if $FOO {', '} else if $BAR {'], profile_data), (['if $FOO {', '} else {'], profile_data)),
        ((['if $FOO and $BAR {'], profile_data),            (['if $FOO and $BAR and defined $BAZ{'], profile_data)),
    )

    @classmethod
    def setUpClass(cls):
        super(TestConditionalEquality, cls).setUpClass()
        cls.active_profiles = ProfileList()
        (profile, hat, prof_storage) = ProfileStorage.parse('profile foo {', cls.filename, 0, False, False)
        cls.profile_data[profile] = prof_storage
        cls.profile_data[profile]['file'].add(FileRule.create_instance('/bin/false rix,'))
        cls.profile_data[profile]['file'].add(FileRule.create_instance('/** w,'))
        (profile, hat, prof_storage) = ProfileStorage.parse('^bar {', cls.filename, 0, profile, None)
        profname = combine_profname((profile, hat))
        cls.profile_data[profname] = prof_storage
        cls.profile_data[profname]['in_cond'] = True
        cls.profile_data[profname]['file'].add(FileRule.create_instance('/bin/true rix,'))
        cls.active_profiles.add_rule(cls.filename, 'variable', VariableRule.create_instance('@{VAR1} = "test"'))
        cls.active_profiles.add_rule(cls.filename, 'boolean', BooleanRule.create_instance('$FOO=true'))
        cls.active_profiles.add_rule(cls.filename, 'boolean', BooleanRule.create_instance('$BAR=false'))

    def init_cond(self, conds, profile_data):
        block = None
        for cond in conds:
            if cond == conds[0]:
                block = ConditionalBlock(cond, self.active_profiles.files[self.filename])
                block.store_profile_data(profile_data)
            else:
                block.add_conditional(cond, self.active_profiles.files[self.filename])
                block.store_profile_data(profile_data)
        return block

    def _run_test(self, cond1, cond2):
        conditional_block1 = self.init_cond(cond1[0], cond1[1])
        conditional_block2 = self.init_cond(cond2[0], cond2[1])

        self.assertFalse(conditional_block1.is_equal(conditional_block2))
        self.assertFalse(conditional_block1.is_covered(conditional_block2))

    def test_is_equal_condition(self):
        conditional_block1 = self.init_cond(['if $FOO {', '} else if $BAR {', '} else {'], self.profile_data)
        conditional_block2 = self.init_cond(['if $FOO {', '} else {'], self.profile_data)

        self.assertFalse(conditional_block1.is_equal(conditional_block2))
        self.assertFalse(conditional_block1.is_covered(conditional_block2))

    def test_is_equal(self):
        conditional_block1 = ConditionalBlock('if not defined $FOO {', self.active_profiles.files[self.filename])
        conditional_block1.store_profile_data(self.profile_data)

        self.assertTrue(conditional_block1.is_equal(conditional_block1))
        self.assertTrue(conditional_block1.is_covered(conditional_block1))


class TestConditionalException(AATest):
    filename = 'somefile'
    profile_data = {}

    tests = (
        (['if @{VAR1} {', '} else if $BAR {', '} else {'], AppArmorException),  # set variable used in boolean comparison
        (['if $FOO {', '} else if @{VAR} {', '} else {'],  AppArmorException),  # undefined variable in else if
        (['if ${BAZ} {', '} else {'],                      AppArmorException),  # undefined boolean variable with else
        (['if ${VAR} {'],                                  AppArmorException),  # undefined boolean variable
        (['if foo in @{UNDERFINED} {'],                    AppArmorException),  # undefined set variable
    )

    @classmethod
    def setUpClass(cls):
        super(TestConditionalException, cls).setUpClass()
        cls.active_profiles = ProfileList()
        cls.condition_contents = '\n'
        (profile, hat, prof_storage) = ProfileStorage.parse('profile foo {', cls.filename, 0, False, False)
        cls.profile_data[profile] = prof_storage
        cls.active_profiles.add_rule(cls.filename, 'variable', VariableRule.create_instance('@{VAR1} = "test"'))
        cls.active_profiles.add_rule(cls.filename, 'boolean', BooleanRule.create_instance('$FOO=true'))
        cls.active_profiles.add_rule(cls.filename, 'boolean', BooleanRule.create_instance('$BAR=false'))

    def _run_test(self, params, expected):
        conditional_block = None
        with self.assertRaises(expected):
            for condition in params:
                if condition == params[0]:
                    conditional_block = ConditionalBlock(condition, self.active_profiles.files[self.filename])
                    conditional_block.store_profile_data(self.profile_data)
                else:
                    conditional_block.add_conditional(condition, self.active_profiles.files[self.filename])
                    conditional_block.store_profile_data(self.profile_data)

    def test_create_instance_block1(self):
        with self.assertRaises(NotImplementedError):
            ConditionalBlock.create_instance('test')

    def test_create_instance_block2(self):
        with self.assertRaises(NotImplementedError):
            ConditionalBlock._create_instance('test', None)


class TestAppArmorAstException(AATest):
    tests = (
        ('foo = "1"',    AppArmorBug),  # AppArmorAst only supports the actual condition
        ('~ $FOO',       AppArmorBug),  # Invalid unary operator
        ('func(${BAZ})', AppArmorBug),  # Invalid function name
    )

    def _run_test(self, params, expected):
        ast = AppArmorAst(params)
        with self.assertRaises(expected):
            ast.get_clean()
        with self.assertRaises(expected):
            ast.evaluate(None)


class TestMiscClassException(AATest):
    def test_invalid_op_1(self):
        term = Term.create_instance('10')
        op = 'is'
        with self.assertRaises(AppArmorBug):
            CompareCondition(term, op, term)

    def test_invalid_op_2(self):
        term = Term.create_instance('10')
        cond = CompareCondition(term, 'in', term)
        cond.op = 'is'
        with self.assertRaises(AppArmorBug):
            cond.evaluate(None)

    def test_invalid_term(self):
        with self.assertRaises(AppArmorBug):
            Term.create_instance('#')

    def test_invalid_conditional_condition(self):
        with self.assertRaises(AppArmorBug):
            ConditionalRule('invalid', AppArmorAst('defined $BAR'))

    def test_invalid_conditional_ast_tree(self):
        with self.assertRaises(AppArmorBug):
            ConditionalRule(ConditionalRule.IF, 'defined $BAR')

    def test_invalid_conditional_clean(self):
        cond = ConditionalRule(ConditionalRule.IF, AppArmorAst('defined $BAR'))
        cond.condition = 'invalid'
        with self.assertRaises(AppArmorBug):
            cond.get_clean()

    def test_invalid_boolean_defined(self):
        with self.assertRaises(AppArmorBug):
            BooleanCondition(None, Term.create_instance('10'))

    def test_invalid_boolean_varible(self):
        with self.assertRaises(AppArmorBug):
            BooleanCondition('defined', '10')

    def test_invalid_compare_left(self):
        with self.assertRaises(AppArmorBug):
            CompareCondition(None, '==', Term.create_instance('10'))

    def test_invalid_compare_right(self):
        with self.assertRaises(AppArmorBug):
            CompareCondition(Term.create_instance('10'), '==', None)

    def test_invalid_compare_op(self):
        with self.assertRaises(AppArmorBug):
            CompareCondition(Term.create_instance('10'), '~', Term.create_instance('10'))

    def test_invalid_compare(self):
        term = Term.create_instance('10')
        cond = CompareCondition(term, '==', term)
        with self.assertRaises(AppArmorBug):
            cond.compare(cond.op, term.get_set(None), 10)

    def test_invalid_compare_evaluate(self):
        term = InvalidTerm(10)
        cond = CompareCondition(term, '==', term)
        with self.assertRaises(AppArmorBug):
            cond.evaluate(None)


class InvalidTerm(Term):
    def __init__(self, value):
        self.value = value

    def get_set(self, prof_storage):
        return self.value  # not a set


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2024 Canonical, Ltd.
#    Copyright (C) 2025 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple
from common_test import AATest, setup_all_loops

from apparmor.common import AppArmorException, AppArmorBug
from apparmor.translations import init_translation

from apparmor.rule.mount import MountRule, MountConditional

_ = init_translation()


class MountTestParse(AATest):

    tests = (
        #                   Rule                                                     Operation   Filesystem                Options                  Source          Destination     Audit  Deny   Allow  Comment
        ('mount -> **,',                                                    MountRule('mount',   MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  '**',           False, False, False, '')),
        ('mount options=(rw, shared) -> **,',                               MountRule('mount',   MountRule.ALL,            ('=', ('rw', 'shared')), MountRule.ALL,  '**',           False, False, False, '')),
        ('mount options=rw,',                                               MountRule('mount',   MountRule.ALL,            ('=', ('rw')),           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('mount fstype=bpf options=rw bpf -> /sys/fs/bpf/,',                MountRule('mount',   ('=', ['bpf']),           ('=', ('rw')),           'bpf',          '/sys/fs/bpf/', False, False, False, '')),
        ('mount fstype=fuse.obex* options=rw bpf -> /sys/fs/bpf/,',         MountRule('mount',   ('=', ['fuse.obex*']),    ('=', ('rw')),           'bpf',          '/sys/fs/bpf/', False, False, False, '')),
        ('mount fstype=fuse.* options=rw bpf -> /sys/fs/bpf/,',             MountRule('mount',   ('=', ['fuse.*']),        ('=', ('rw')),           'bpf',          '/sys/fs/bpf/', False, False, False, '')),
        ('mount fstype=bpf options=(rw) random_label -> /sys/fs/bpf/,',     MountRule('mount',   ('=', ['bpf']),           ('=', ('rw')),           'random_label', '/sys/fs/bpf/', False, False, False, '')),
        ('mount,',                                                          MountRule('mount',   MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('mount fstype=(ext3, ext4),',                                      MountRule('mount',   ('=', ['ext3', 'ext4']),  MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('mount bpf,',                                                      MountRule('mount',   MountRule.ALL,            MountRule.ALL,           'bpf',          MountRule.ALL,  False, False, False, '')),
        ('mount none,',                                                     MountRule('mount',   MountRule.ALL,            MountRule.ALL,           'none',         MountRule.ALL,  False, False, False, '')),
        ('mount fstype=(procfs) none -> /foo,',                             MountRule('mount',   ('=', ['procfs']),        MountRule.ALL,           'none',         '/foo',         False, False, False, '')),
        ('mount fstype=(ext3, ext4) options=(ro),',                         MountRule('mount',   ('=', ['ext3', 'ext4']),  ('=', ('ro')),           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('mount @{mntpnt},',                                                MountRule('mount',   MountRule.ALL,            MountRule.ALL,           '@{mntpnt}',    MountRule.ALL,  False, False, False, '')),
        ('mount /a,',                                                       MountRule('mount',   MountRule.ALL,            MountRule.ALL,           '/a',           MountRule.ALL,  False, False, False, '')),
        ('mount "/a space",',                                               MountRule('mount',   MountRule.ALL,            MountRule.ALL,           '/a space',     MountRule.ALL,  False, False, False, '')),
        ('mount fstype=ext3,ext4 /a -> /b,',                                MountRule('mount',   ('=', ['ext3', 'ext4']),  MountRule.ALL,           '/a',           '/b',           False, False, False, '')),
        ('mount fstype=(ext3, ext4) /a -> /b,',                             MountRule('mount',   ('=', ['ext3', 'ext4']),  MountRule.ALL,           '/a',           '/b',           False, False, False, '')),
        ('mount fstype=(ext3, ext4) /a -> "/bar space",',                   MountRule('mount',   ('=', ['ext3', 'ext4']),  MountRule.ALL,           '/a',           '/bar space',   False, False, False, '')),
        ('mount fstype=(ext3, ext4) options=(ro, sync) /a -> /b,',          MountRule('mount',   ('=', ['ext3', 'ext4']),  ('=', ('ro', 'sync')),   '/a',           '/b',           False, False, False, '')),
        ('mount fstype=(ext3, ext4) options=(ro, sync) /a -> /b, #cmt',     MountRule('mount',   ('=', ['ext3', 'ext4']),  ('=', ('ro', 'sync')),   '/a',           '/b',           False, False, False, ' #cmt')),
        ('mount fstype=({ext3,ext4}) options in (ro, sync) /a -> /b,',      MountRule('mount',   ('=', ['ext3', 'ext4']),  ('in', ('ro', 'sync')),  '/a',           '/b',           False, False, False, '')),
        ('mount fstype=(  {ext3,ext4}  ) options in (ro, sync) /a -> /b,',  MountRule('mount',   ('=', ['ext3', 'ext4']),  ('in', ('ro', 'sync')),  '/a',           '/b',           False, False, False, '')),
        ('mount fstype in (ext3, ext4) options=(ro, sync) /a -> /b, #cmt',  MountRule('mount',   ('in', ['ext3', 'ext4']), ('=', ('ro', 'sync')),   '/a',           '/b',           False, False, False, ' #cmt')),
        ('mount fstype in (ext3, ext4) option in (ro, sync) /a, #cmt',      MountRule('mount',   ('in', ['ext3', 'ext4']), ('in', ('ro', 'sync')),  '/a',           MountRule.ALL,  False, False, False, ' #cmt')),
        ('mount fstype=(ext3, ext4) option=(ro, sync) /a -> /b, #cmt',      MountRule('mount',   ('=', ['ext3', 'ext4']),  ('=', ('ro', 'sync')),   '/a',           '/b',           False, False, False, ' #cmt')),
        ('mount fstype=fuse.revokefs-fuse options=(nosuid,nodev,rw) revokefs-fuse -> /foo-*/**/,',
                                                                            MountRule('mount',  ('=', ['fuse.revokefs-fuse']), ('=', ('nosuid', 'nodev', 'rw')),                                                # noqa: E127
                                                                                                                                                    'revokefs-fuse', '/foo-*/**/',  False, False, False, '')),  # noqa: E127
        ('mount options=(rw, rbind) {,/usr}/lib{,32,64,x32}/modules/ -> /tmp/snap.rootfs_*{,/usr}/lib/modules/,',
                                                                            MountRule('mount',   MountRule.ALL,            ('=', ('rw', 'rbind')),  '{,/usr}/lib{,32,64,x32}/modules/',  # noqa: E127
                                                                                                                                                                   '/tmp/snap.rootfs_*{,/usr}/lib/modules/',  # noqa: E127
                                                                                                                                                                                    False, False, False, '')),  # noqa: E127
        ('mount options=(runbindable, rw) -> /,',                           MountRule('mount',   MountRule.ALL,            ('=', ['runbindable', 'rw']), MountRule.ALL, '/',        False, False, False, '')),
        ('mount "" -> /,',                                                  MountRule('mount',   MountRule.ALL,            MountRule.ALL,           '',             '/',            False, False, False, '')),

        ('mount options=(ro) options=(rw) fstype=ext4 -> /dest,',           MountRule('mount',   ('=', ['ext4']),          [('=', ('ro')), ('=', ('rw'))],                                                      # noqa: E127
                                                                                                                                                    MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127
        ('mount options=(ro) fstype=ext4 options=(rw) /src -> /dest,',      MountRule('mount',   ('=', ['ext4']),          [('=', ('ro')), ('=', ('rw'))],                                                      # noqa: E127
                                                                                                                                                    '/src',        '/dest',         False, False, False, '')),  # noqa: E127
        ('mount options in (ro) options in (rw) fstype=ext4 -> /dest,',     MountRule('mount',   ('=', ['ext4']),          [('in', ('ro')), ('in', ('rw'))],                                                    # noqa: E127
                                                                                                                                                    MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127
        ('mount options in (ro) fstype=ext4 options in (rw) -> /dest,',     MountRule('mount',   ('=', ['ext4']),          [('in', ('ro')), ('in', ('rw'))],                                                    # noqa: E127
                                                                                                                                                    MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127
        ('mount options = (ro) options in (rw) fstype=ext4 -> /dest,',      MountRule('mount',   ('=', ['ext4']),          [('=', ('ro')), ('in', ('rw'))],                                                     # noqa: E127
                                                                                                                                                    MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127
        ('mount options = (ro) fstype=ext4 options in (rw) -> /dest,',      MountRule('mount',   ('=', ['ext4']),          [('=', ('ro')), ('in', ('rw'))],                                                     # noqa: E127
                                                                                                                                                    MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127

        ('mount options=(ro) fstype=ext3 fstype=ext4 -> /dest,',            MountRule('mount',   [('=', ['ext3']), ('=', ['ext4'])],
                                                                                                                           ('=', ('ro')),           MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127
        ('mount fstype=ext3 options=(ro) fstype=ext4 src -> /dest,',        MountRule('mount',   [('=', ['ext3']), ('=', ['ext4'])],
                                                                                                                           ('=', ('ro')),           'src',         '/dest',         False, False, False, '')),  # noqa: E127
        ('mount options=(ro) fstype in (ext3) fstype in (ext4) -> /dest,',  MountRule('mount',   [('in', ['ext3']), ('in', ['ext4'])],
                                                                                                                           ('=', ('ro')),           MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127
        ('mount fstype in (ext3) options=(ro) fstype in (ext4) -> /dest,',  MountRule('mount',   [('in', ['ext3']), ('in', ['ext4'])],
                                                                                                                           ('=', ('ro')),           MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127
        ('mount options=(ro) fstype in (ext3) fstype=(ext4) -> /dest,',     MountRule('mount',   [('in', ['ext3']), ('=', ['ext4'])],
                                                                                                                           ('=', ('ro')),           MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127
        ('mount fstype = (ext3) options=(ro) fstype in ext4 -> /dest,',     MountRule('mount',   [('=', ['ext3']), ('in', ['ext4'])],
                                                                                                                           ('=', ('ro')),           MountRule.ALL, '/dest',         False, False, False, '')),  # noqa: E127

        ('umount,',                                                         MountRule('umount',  MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('umount fstype=ext3,',                                             MountRule('umount',  ('=', ['ext3']),          MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('umount /a,',                                                      MountRule('umount',  MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  '/a',           False, False, False, '')),
        ('umount "/foo/b a r",',                                            MountRule('umount',  MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  '/foo/b a r',   False, False, False, '')),

        ('remount,',                                                        MountRule('remount', MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('remount fstype=ext4,',                                            MountRule('remount', ('=', ['ext4']),          MountRule.ALL,           MountRule.ALL,  MountRule.ALL,  False, False, False, '')),
        ('remount /b,',                                                     MountRule('remount', MountRule.ALL,            MountRule.ALL,           MountRule.ALL,  '/b',           False, False, False, '')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(MountRule.match(rawrule))
        obj = MountRule.create_instance(rawrule)
        expected.raw_rule = rawrule.strip()
        self.assertTrue(obj.is_equal(expected, True), f'\n  {rawrule}   expected,\n  {obj.get_clean()}   returned by obj.get_clean()\n  {expected.get_clean()}   returned by expected.get_clean()')

    def test_valid_mount_changing_propagation(self):
        # Rules changing propagation type can either specify a source or a dest (these are equivalent for apparmor_parser in this specific case) but not both.
        MountRule('mount', MountRule.ALL, ('=', ('runbindable')), '/foo', MountRule.ALL)
        MountRule('mount', MountRule.ALL, ('=', ('runbindable')), MountRule.ALL, '/foo')

    def test_valid_bind_mount(self):
        # Fstype must remain empty in bind rules
        MountRule('mount', MountRule.ALL, ('=', ('bind')), '/foo', MountRule.ALL)
        MountRule('mount', MountRule.ALL, ('=', ('bind')), MountRule.ALL, '/bar')
        MountRule('mount', MountRule.ALL, ('=', ('bind')), '/foo', '/bar')


class MountTestParseInvalid(AATest):
    tests = (
        #                                    exception          matches regex
        ('mount fstype=,',                  (AppArmorException, True)),
        ('mount fstype=(),',                (AppArmorException, True)),
        ('mount options=(),',               (AppArmorException, True)),
        ('mount option=(invalid),',         (AppArmorException, True)),
        ('mount option=(ext3ext4),',        (AppArmorException, True)),
        ('priority=-1042 umount,',          (AppArmorException, True)),
        ('mount fstype=({unclosed_regex),', (AppArmorException, True)),  # invalid AARE
        ('mount fstype=({closed}twice}),',  (AppArmorException, True)),  # invalid AARE
        ('foo,',                            (AppArmorException, False)),
        ('priority=a mount,',               (AppArmorException, False)),
        ('priority=a umount,',              (AppArmorException, False)),
        ('priority=a unmount,',             (AppArmorException, False)),
        ('priority=a remount,',             (AppArmorException, False)),
    )

    def _run_test(self, rawrule, expected):
        self.parseInvalidRule(MountRule, rawrule, expected)


class MountTestInvalid(AATest):
    def test_invalid_priority_1(self):
        with self.assertRaises(TypeError):
            MountRule('mount', MountRule.ALL, MountRule.ALL, MountRule.ALL, MountRule.ALL, priority=MountRule.ALL)

    def test_invalid_priority_2(self):
        with self.assertRaises(AppArmorException):
            MountRule('mount', MountRule.ALL, MountRule.ALL, MountRule.ALL, MountRule.ALL, priority='invalid')

    def test_invalid_fstype_equals_or_in(self):
        with self.assertRaises(AppArmorBug):
            MountRule('mount', ('ext3', 'ext4'), MountRule.ALL, MountRule.ALL, MountRule.ALL)  # fstype[0] should be '=' or 'in'

    def test_invalid_fstype_aare_2(self):
        fslists = [
            ['invalid_{_regex'],
            ['ext4', 'invalid_}_regex'],
            ['ext4', '{invalid} {regex}']
        ]
        for fslist in fslists:
            with self.assertRaises(AppArmorException):
                MountRule('mount', ('=', fslist), MountRule.ALL, MountRule.ALL, MountRule.ALL)

    def test_invalid_options_equals_or_in(self):
        with self.assertRaises(AppArmorBug):
            MountRule('mount', MountRule.ALL, ('rbind', 'rw'), MountRule.ALL, MountRule.ALL)  # fstype[0] should be '=' or 'in'

    def test_invalid_options_keyword(self):
        with self.assertRaises(AppArmorException):
            MountRule('mount', MountRule.ALL, ('=', 'invalid'), MountRule.ALL, MountRule.ALL)  # fstype[0] should be '=' or 'in'

    def test_invalid_umount_with_source(self):
        with self.assertRaises(AppArmorException):
            MountRule('umount', MountRule.ALL, MountRule.ALL, '/foo', MountRule.ALL)  # Umount and remount shall not have a source

    def test_invalid_remount_with_source(self):
        with self.assertRaises(AppArmorException):
            MountRule('remount', MountRule.ALL, MountRule.ALL, '/foo', MountRule.ALL)

    def test_invalid_mount_changing_propagation_1(self):
        # Rules changing propagation type can either specify a source or a dest (these are equivalent for apparmor_parser in this specific case) but not both.
        with self.assertRaises(AppArmorException):
            MountRule('mount', MountRule.ALL, ('=', ('runbindable')), '/foo', '/bar')

    def test_invalid_mount_changing_propagation_2(self):
        # Rules changing propagation type cannot specify a fstype.
        with self.assertRaises(AppArmorException):
            MountRule('mount', ('=', ('ext4')), ('=', ('runbindable')), MountRule.ALL, '/foo')

    def test_invalid_bind_mount(self):
        # Bind mount rules cannot specify a fstype.
        with self.assertRaises(AppArmorException):
            MountRule('mount', ('=', ('ext4')), ('=', ('bind')), MountRule.ALL, '/foo')

    def test_invalid_cond_type(self):
        # cond_type cannot be None if all_values is False
        with self.assertRaises(AppArmorBug):
            obj = MountConditional('name', [], False, '=', cond_type=None)
            obj.is_covered(obj)


class MountTestIsEqual(AATest):
    def test_diff_non_mountrule(self):
        exp = namedtuple('exp', ('audit', 'deny', 'priority'))
        obj = MountRule('mount', ('=', ['ext4']), MountRule.ALL, MountRule.ALL, MountRule.ALL)
        with self.assertRaises(AppArmorBug):
            obj.is_equal(exp(False, False, None), False)

    def test_diff_fstype(self):
        obj1 = MountRule('mount', ('=', ['ext4']), MountRule.ALL, MountRule.ALL, MountRule.ALL)
        obj2 = MountRule('mount', MountRule.ALL, MountRule.ALL, MountRule.ALL, MountRule.ALL)
        self.assertFalse(obj1.is_equal(obj2, False))

    def test_diff_source(self):
        obj1 = MountRule('mount', MountRule.ALL, MountRule.ALL, '/foo', MountRule.ALL)
        obj2 = MountRule('mount', MountRule.ALL, MountRule.ALL, '/bar', MountRule.ALL)
        self.assertFalse(obj1.is_equal(obj2, False))


class MountTestGlob(AATest):
    def test_glob(self):
        globList = [(
            'mount options=(bind, rw) /home/user/Downloads/ -> /mnt/a/,',
            'mount options=(bind, rw) /home/user/Downloads/,',
            'mount options=(bind, rw) /home/user/*/,',
            'mount options=(bind, rw) /home/**/,',
            'mount options=(bind, rw),',
            'mount,',
            'mount,',
        )]
        for globs in globList:
            for i in range(len(globs) - 1):
                rule = MountRule.create_instance(globs[i])
                rule.glob()
                self.assertEqual(rule.get_clean(), globs[i + 1])


class MountTestClean(AATest):
    tests = (
        #  raw rule                                                  clean rule
        ('     mount                                                                            ,    # foo  ', 'mount, # foo'),
        ('     mount                                fstype  =  (  sysfs  )                      ,           ', 'mount fstype=sysfs,'),
        ('     mount                                fstype  =  (  sysfs  ,  procfs  )           ,           ', 'mount fstype=procfs,sysfs,'),
        ('     mount                                fstype  =     sysfs,procfs                  ,           ', 'mount fstype=procfs,sysfs,'),
        ('     mount                                fstype  =  ext{3,4}                         ,           ', 'mount fstype=ext{3,4},'),
        ('     mount                                fstype  =  ext3                             ,           ', 'mount fstype=ext3,'),
        ('     mount                                fstype  =  @{foo}                           ,           ', 'mount fstype=@{foo},'),
        ('     mount  options  =  (  rw  )                                                      ,           ', 'mount options=(rw),'),
        ('     mount  options  =  (  rw , noatime  )                                            ,           ', 'mount options=(noatime, rw),'),
        ('     mount                                fstype  in (  sysfs  )                      ,           ', 'mount fstype in (sysfs),'),
        ('     mount                                fstype  in (  sysfs  ,  procfs  )           ,           ', 'mount fstype in (procfs, sysfs),'),
        ('     mount  options  in (  rw  )                                                      ,           ', 'mount options in (rw),'),
        ('     mount  options  in (  rw , noatime  )                                            ,           ', 'mount options in (noatime, rw),'),
        ('     mount     none -> /foo                                                           ,           ', 'mount none -> /foo,'),
        ('     mount     ""   -> /foo                                                           ,           ', 'mount "" -> /foo,'),
        ('     mount  "/f /b" -> "/foo bar"                                                     ,           ', 'mount "/f /b" -> "/foo bar",'),
        ('     umount                                                                           ,           ', 'umount,'),
        ('     umount                                                            /foo           ,           ', 'umount /foo,'),
        ('     remount                                                                          ,           ', 'remount,'),
        ('     remount                                                           /foo           ,           ', 'remount /foo,'),
        (' deny     mount                           fstype  =  (  sysfs  )                      ,           ', 'deny mount fstype=sysfs,'),
        ('  deny   umount                                                        /foo           ,           ', 'deny umount /foo,'),
        ('   deny  remount                                                                      ,           ', 'deny remount,'),
        ('priority =1 mount     ""   -> /foo                                                    ,           ', 'priority=1 mount "" -> /foo,'),
        ('priority=0 audit  mount  "/f /b" -> "/foo bar"                                        ,           ', 'priority=0 audit mount "/f /b" -> "/foo bar",'),
        (' priority  =  +10   umount                                                            ,           ', 'priority=10 umount,'),
        (' priority=-2    deny    umount                                         /foo           ,           ', 'priority=-2 deny umount /foo,'),
        ('priority= 32    audit  deny     remount                                               ,           ', 'priority=32 audit deny remount,'),
        (' priority = -32    remount                                             /foo           ,           ', 'priority=-32 remount /foo,'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(MountRule.match(rawrule))
        obj = MountRule.create_instance(rawrule)
        clean = obj.get_clean()
        cleanspc = obj.get_clean(1)
        raw = obj.get_raw()

        self.assertEqual(expected, clean, 'unexpected clean rule')
        self.assertEqual("  " + expected, cleanspc, 'unexpected clean rule with leading spaces')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')


class MountTestFsType(AATest):
    tests = (
        #  raw fstype       clean fs type
        ('=ext3',           '=ext3'),
        ('=ext{3,4}',       '=ext{3,4}'),
        ('=(ext{3,4})',     '=ext{3,4}'),
        ('=ext3,ext4',      '=ext3,ext4'),
        ('=(ext3,ext4)',    '=ext3,ext4'),
        ('={ext3,ext4}',    '=ext3,ext4'),
        ('=(ext3, ext4)',   '=ext3,ext4'),
        ('={ext3, ext4}',   '=ext3,ext4'),
        ('in ext3',         ' in (ext3)'),
        ('in ext3,ext4',    ' in (ext3, ext4)'),
        ('in (ext3,ext4)',  ' in (ext3, ext4)'),
        ('in {ext3,ext4}',  ' in (ext3, ext4)'),
        ('in (ext3, ext4)', ' in (ext3, ext4)'),
        ('in {ext3, ext4}', ' in (ext3, ext4)'),
    )

    def _run_test(self, fstype, expected):
        clean = MountRule.create_instance("mount fstype " + fstype + ",").get_clean()
        self.assertEqual("mount fstype" + expected + ",", clean)


class MountLogprofHeaderTest(AATest):
    tests = (
        ('mount,',                                                   [_('Operation'), _('mount'), _('Fstype'), _('ALL'),              _('Options'), _('ALL'),              _('Source'), _('ALL'), _('Destination'), _('ALL')]),
        ('mount options=(ro, nosuid) /a,',                           [_('Operation'), _('mount'), _('Fstype'), _('ALL'),              _('Options'), ('=', _('nosuid ro')), _('Source'), _('/a'),  _('Destination'), _('ALL')]),
        ('mount fstype=(ext3, ext4) options=(ro, nosuid) /a -> /b,', [_('Operation'), _('mount'), _('Fstype'), ('=', _('ext3 ext4')), _('Options'), ('=', _('nosuid ro')), _('Source'), _('/a'),  _('Destination'), _('/b')])
    )

    def _run_test(self, params, expected):
        obj = MountRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


class MountIsCoveredTest(AATest):
    def test_is_covered(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), ('=', ('ro')), '/foo/b*', '/b*')
        tests = [
            ('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')),   '/foo/b', '/bar'),
            ('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')),   '/foo/bar', '/b'),
            ('mount', ('=', ['ext3', 'ext4']), [('=', ('ro'))], '/foo/bar', '/b')
        ]
        for test in tests:
            self.assertTrue(obj.is_covered(MountRule(*test)))
            self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_covered_fs_source(self):
        obj = MountRule('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        self.assertTrue(obj.is_covered(MountRule('mount', ('=', ['ext3']), ('=', ('ro')), 'tmpfs', MountRule.ALL)))
        self.assertFalse(obj.is_equal(MountRule('mount', ('=', ['ext3']), ('=', ('ro')), 'tmpfs', MountRule.ALL)))

    def test_is_covered_aare_1(self):
        obj = MountRule('mount', ('=', ['sys*', 'fuse.*']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        tests = [
            ('mount', ('=', ['sysfs', 'fuse.s3fs']), ('=', ('ro')), 'tmpfs', MountRule.ALL),
            ('mount', ('=', ['sysfs', 'fuse.jmtpfs', 'fuse.s3fs', 'fuse.obexfs', 'fuse.obexautofs', 'fuse.fuseiso']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        ]
        for test in tests:
            self.assertTrue(obj.is_covered(MountRule(*test)))
            self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_covered_aare_2(self):
        obj = MountRule('mount', ('=', ['ext{3,4}', '{cgroup*,fuse.*}']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        tests = [
            ('mount', ('=', ['ext3']), ('=', ('ro')), 'tmpfs', MountRule.ALL),
            ('mount', ('=', ['ext3', 'ext4', 'cgroup', 'cgroup2', 'fuse.jmtpfs', 'fuse.s3fs', 'fuse.obexfs', 'fuse.obexautofs', 'fuse.fuseiso']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        ]
        for test in tests:
            self.assertTrue(obj.is_covered(MountRule(*test)))
            self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_covered_options_diff_operator(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), ('=', ('ro')), '/foo/b*', '/b*')
        test_obj = MountRule('mount', ('=', ('ext3', 'ext4')), ('in', ('ro')), '/foo/b*', '/b*')
        self.assertFalse(obj.is_covered(test_obj), '\n' + test_obj.get_clean() + '\n should not be covered by\n' + obj.get_clean())

    def test_is_covered_options_all(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), MountRule.ALL, '/foo/b*', '/b*')
        test_obj = MountRule('mount', ('=', ('ext3', 'ext4')), ('in', ('ro')), '/foo/b*', '/b*')
        self.assertTrue(obj.is_covered(test_obj), '\n' + test_obj.get_clean() + '\n should be covered by\n' + obj.get_clean())

    def test_is_covered_options_true(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), [('in', ('ro')), ('=', ('rw', 'noexec')), ('in', ('ro', 'nosuid'))], '/foo/b*', '/b*')
        test_obj = MountRule('mount', ('=', ('ext3', 'ext4')), [('=', ('rw', 'noexec')), ('in', ('ro', 'nosuid'))], '/foo/b*', '/b*')
        self.assertTrue(obj.is_covered(test_obj), '\n' + test_obj.get_clean() + '\n should be covered by\n' + obj.get_clean())

    def test_is_equal_options_diff_operator(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), ('=', ('ro')), '/foo/b*', '/b*')
        test_obj = MountRule('mount', ('=', ('ext3', 'ext4')), ('in', ('ro')), '/foo/b*', '/b*')
        self.assertFalse(obj.is_equal(test_obj, True), '\n' + test_obj.get_clean() + '\n should not be equal to\n' + obj.get_clean())

    def test_is_equal_options_diff_options(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), [('in', ('ro')), ('=', ('nosuid', 'noexec'))], '/foo/b*', '/b*')
        test_obj = MountRule('mount', ('=', ('ext3', 'ext4')), ('in', ('ro')), '/foo/b*', '/b*')
        self.assertFalse(obj.is_equal(test_obj, True), '\n' + test_obj.get_clean() + '\n should not be equal to\n' + obj.get_clean())

    def test_is_equal_options_true(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), [('in', ('ro')), ('=', ('nosuid', 'noexec'))], '/foo/b*', '/b*')
        test_obj = MountRule('mount', ('=', ('ext3', 'ext4')), [('in', ('ro')), ('=', ('nosuid', 'noexec'))], '/foo/b*', '/b*')
        self.assertTrue(obj.is_equal(test_obj, True), '\n' + test_obj.get_clean() + '\n should be equal to\n' + obj.get_clean())

    def test_eq_cond_diff_type(self):
        obj = MountConditional('options', ['ro'], False, 'in', 'aare')
        test_obj = MountConditional('options', ['ro'], False, 'in', 'list')
        self.assertFalse(test_obj == obj, 'cond_types should be different')

    def test_eq_cond_diff_instance(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), [('in', ('ro')), ('=', ('nosuid', 'noexec'))], '/foo/b*', '/b*')
        test_obj = MountConditional('options', ['ro'], False, 'in', 'list')
        self.assertFalse(test_obj == obj, 'Incompatible comparison')

    def test_covered_cond_diff_instance(self):
        obj = MountRule('mount', ('=', ('ext3', 'ext4')), [('in', ('ro')), ('=', ('nosuid', 'noexec'))], '/foo/b*', '/b*')
        test_obj = MountConditional('options', ['ro'], False, 'in', 'list')
        self.assertFalse(test_obj.is_covered(obj), 'Incompatible comparison')

    def test_covered_cond_diff_name(self):
        obj = MountConditional('foo', ['ro'], False, '=', 'list')
        test_obj = MountConditional('options', ['ro'], False, '=', 'list')
        self.assertFalse(test_obj.is_covered(obj), 'Incompatible comparison')

    def test_is_notcovered(self):
        obj = MountRule('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), '/foo/b*', '/b*')
        tests = [
            ('mount',   ('in', ['ext3', 'ext4']),   ('=',  ('ro')), '/foo/bar',     '/bar'),
            ('mount',   ('=', ['procfs', 'ext4']),  ('=',  ('ro')), '/foo/bar',     '/bar'),
            ('mount',   ('=', ['ext3']),            ('=',  ('rw')), '/foo/bar',     '/bar'),
            ('mount',   ('=', ['ext3', 'ext4']),    MountRule.ALL, '/foo/b*',      '/bar'),
            ('mount',   MountRule.ALL,              ('=',  ('ro')), '/foo/b*',      '/bar'),
            ('mount',   ('=', ['ext3', 'ext4']),    ('=',  ('ro')), '/invalid/bar', '/bar'),
            ('umount',  MountRule.ALL,              MountRule.ALL, MountRule.ALL,  '/bar'),
            ('remount', MountRule.ALL,              MountRule.ALL, MountRule.ALL,  '/bar'),
            ('mount',   ('=', ['ext3', 'ext4']),    ('=',  ('ro')), 'tmpfs',        '/bar'),
            ('mount',   ('=', ['ext3', 'ext4']),    ('=',  ('ro')), '/foo/b*',      '/invalid'),
            ('mount',   ('=', ['ext3']),            ('in', ('ro')), '/foo/bar',     '/bar'),
            ('mount',   ('=', ['ext3']),            [('in', ('ro')), ('=', ('rw'))], '/foo/bar', '/bar'),
            ('mount',   ('=', ['ext3']),            [('=', ('ro')), ('in', ('rw'))], '/foo/bar', '/bar'),
            ('mount',   ('=', ['ext3']),            [('=', ('ro', 'nosuid')), ('in', ('rw'))], '/foo/bar', '/bar'),
        ]
        for test in tests:
            self.assertFalse(obj.is_covered(MountRule(*test)))
            self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_not_covered_fs_source(self):
        obj = MountRule('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), 'tmpfs', MountRule.ALL)
        test = ('mount', ('=', ['ext3', 'ext4']), ('=', ('ro')), 'procfs', MountRule.ALL)
        self.assertFalse(obj.is_covered(MountRule(*test)))
        self.assertFalse(obj.is_equal(MountRule(*test)))

    def test_is_not_covered_fs_options(self):
        obj = MountRule('mount', MountRule.ALL, ('=', ('ro')), 'tmpfs', MountRule.ALL)
        test = ('mount', MountRule.ALL, ('=', ('rw')), 'procfs', MountRule.ALL)
        self.assertFalse(obj.is_covered(MountRule(*test)))
        self.assertFalse(obj.is_equal(MountRule(*test)))


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2021 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import unittest

from apparmor.common import AppArmorBug
from apparmor.notify import get_last_login_timestamp, get_last_login_timestamp_wtmp, sane_timestamp, get_event_special_type, set_userns_special_profile
from apparmor.logparser import ReadLog
from common_test import AATest, setup_all_loops


class TestGet_last_login_timestamp(AATest):
    tests = (
        #  wtmp file         lastlog2 db         user           expected login timestamp
        (('wtmp-x86_64',    'lastlog2.db',     'root'),         1723749426),  # Thu Aug 15 19:17:06 UTC 2024
        (('wtmp-x86_64',    'lastlog2.db',     'whoever'),      0),
        (('wtmp-x86_64',    'lastlog2.db',     'cb'),           1726995194),  # Sun Sep 22 08:53:14 UTC 2024
        (('wtmp-x86_64',    'lastlog2.db',     'sddm'),         1721084423),  # Mon Jul 15 23:00:23 UTC 2024

        (('wtmp-x86_64',    'does-not-exist',   'root'),        1635070346),  # Sun Oct 24 12:12:26 CEST 2021
        (('wtmp-x86_64',    'does-not-exist',   'whoever'),     0),

        (('wtmp-s390x',     'lastlog2.db',      'root'),        1723749426),  # Thu Aug 15 19:17:06 UTC 2024
        (('wtmp-s390x',     'lastlog2.db',      'whoever'),     0),
        (('wtmp-s390x',     'does-not-exist',   'linux1'),      1626368772),  # Thu Jul 15 19:06:12 CEST 2021
        (('wtmp-s390x',     'does-not-exist',   'whoever'),     0),

        (('wtmp-aarch64',   'lastlog2.db',      'whoever'),     0),
        (('wtmp-aarch64',   'does-not-exist',   'guillaume'),   1611562789),  # Mon Jan 25 09:19:49 CET 2021
        (('wtmp-aarch64',   'does-not-exist',   'whoever'),     0),

        (('wtmp-truncated', 'does-not-exist',   'root'),        0),
        (('wtmp-truncated', 'does-not-exist',   'whoever'),     0),
    )

    def _run_test(self, params, expected):
        wtmpdb, lastlog2_db, user = params
        wtmpdb = 'wtmp-examples/' + wtmpdb
        lastlog2_db = 'wtmp-examples/' + lastlog2_db
        self.assertEqual(get_last_login_timestamp(user, wtmpdb, lastlog2_db), expected)

    def test_date_1999(self):
        with self.assertRaises(AppArmorBug):
            # wtmp-x86_64-past is hand-edited to Thu Dec 30 00:00:00 CET 1999, which is outside the expected data range
            get_last_login_timestamp('root', 'wtmp-examples/wtmp-x86_64-past', 'wtmp-examples/does-not-exist')

    def test_nonexistent_file(self):
        self.assertEqual(get_last_login_timestamp('root', filename='/does/not/exist', lastlog2_db='/also/does/not/exist'), 0)


class TestSane_timestamp(AATest):
    tests = (
        (2524704400, False),  # Sun Jan  2 03:46:40 CET 2050
        (944780400, False),  # Fri Dec 10 00:00:00 CET 1999
        (1635026400, True),  # Sun Oct 24 00:00:00 CEST 2021
    )

    def _run_test(self, params, expected):
        self.assertEqual(sane_timestamp(params), expected)


class TestGet_last_login_timestamp_wtmp(AATest):
    tests = (
        (('wtmp-x86_64',    'root'),      1635070346),  # Sun Oct 24 12:12:26 CEST 2021
        (('wtmp-x86_64',    'whoever'),   0),
        (('wtmp-s390x',     'root'),      1626368763),  # Thu Jul 15 19:06:03 CEST 2021
        (('wtmp-s390x',     'linux1'),    1626368772),  # Thu Jul 15 19:06:12 CEST 2021
        (('wtmp-s390x',     'whoever'),   0),
        (('wtmp-aarch64',   'guillaume'), 1611562789),  # Mon Jan 25 09:19:49 CET 2021
        (('wtmp-aarch64',   'whoever'),   0),
        (('wtmp-truncated', 'root'),      0),
        (('wtmp-truncated', 'whoever'),   0),
    )

    def _run_test(self, params, expected):
        filename, user = params
        filename = 'wtmp-examples/' + filename
        self.assertEqual(get_last_login_timestamp_wtmp(user, filename), expected)

    def test_date_1999(self):
        with self.assertRaises(AppArmorBug):
            # wtmp-x86_64-past is hand-edited to Thu Dec 30 00:00:00 CET 1999, which is outside the expected data range
            get_last_login_timestamp_wtmp('root', 'wtmp-examples/wtmp-x86_64-past')


class TestEventSpecialType(AATest):
    userns_special_profiles = set_userns_special_profile(['unconfined', 'unprivileged_userns', 'unpriv_.*'])
    parser = ReadLog('', '', '')
    tests = (
        ('[  176.385388] audit: type=1400 audit(1666891380.570:78): apparmor="DENIED" operation="userns_create" class="namespace" profile="/usr/bin/bwrap-userns-restrict" pid=1785 comm="userns_child_ex" requested="userns_create" denied="userns_create"',                                                                                           'normal'),
        ('[  839.488169] audit: type=1400 audit(1752065668.819:208): apparmor="DENIED" operation="userns_create" class="namespace" info="Userns create restricted - failed to find unprivileged_userns profile" error=-13 profile="unconfined" pid=12124 comm="unshare" requested="userns_create" denied="userns_create" target="unprivileged_userns"', 'userns_denied'),
        ('[  429.272003] audit: type=1400 audit(1720613712.153:168): apparmor="AUDIT" operation="userns_create" class="namespace" info="Userns create - transitioning profile" profile="unconfined" pid=5630 comm="unshare" requested="userns_create" target="unprivileged_userns" execpath="/usr/bin/unshare"',                                        'userns_change_profile'),
        ('[   52.901383] audit: type=1400 audit(1752064882.228:82): apparmor="DENIED" operation="capable" class="cap" profile="unprivileged_userns" pid=6700 comm="electron" capability=21  capname="sys_admin"',                                                                                                                                       'userns_capable'),
        ('Jul 31 17:11:16 dbusdev-saucy-amd64 dbus[1692]: apparmor="DENIED" operation="dbus_bind"  bus="session" name="com.apparmor.Test" mask="bind" pid=2940 profile="/tmp/apparmor-2.8.0/tests/regression/apparmor/dbus_service"',                                                                                                                   'normal'),
        ('[103975.623545] audit: type=1400 audit(1481284511.494:2807): apparmor="DENIED" operation="change_onexec" info="no new privs" error=-1 namespace="root//lxd-tor_<var-lib-lxd>" profile="unconfined" name="system_tor" pid=18593 comm="(tor)" target="system_tor"',                                                                             'userns_change_profile'),
        ('[78661.551820] audit: type=1400 audit(1752661047.170:350): apparmor="DENIED" operation="capable" class="cap" profile="unpriv_bwrap" pid=1412550 comm="node" capability=21  capname="sys_admin"',                                                                                                                                              'userns_capable'),
    )

    def _run_test(self, ev, expected):
        parsed_event = self.parser.parse_event(ev)
        r = self.parser.create_rule_from_ev(parsed_event)
        self.assertIsNotNone(r)

        real_type = get_event_special_type(parsed_event, self.userns_special_profiles)
        self.assertEqual(expected, real_type,
                         "ev {}: {} != {}".format(ev, expected, real_type))

    def test_invalid(self):
        ev = 'type=AVC msg=audit(1333698107.128:273917): apparmor="DENIED" operation="recvmsg" parent=1596 profile="unprivileged_userns" pid=1875 comm="nc" laddr=::ffff:127.0.0.1 lport=2048 faddr=::ffff:127.0.0.1 fport=59180 family="inet6" sock_type="stream" protocol=6'
        parsed_event = self.parser.parse_event(ev)
        parsed_event['comm'] = 'something'  # Artificially crafted invalid event
        with self.assertRaises(AppArmorBug):
            get_event_special_type(parsed_event, self.userns_special_profiles)


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2025 Maxime Bélair <maxime.belair@canonical.com>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import os
import sys
import unittest
import subprocess

import apparmor.aa as aa
from apparmor.common import cmd
from common_test import AATest, setup_aa, setup_all_loops


class AAShowUsageTest(AATest):

    def test_help_contents(self):
        """Test output of help text"""

        expected_return_code = 0

        expected_output_1 = \
'''usage: aa-show-usage [-h] [-s {all,used,unused}] [-j] [-d DIR]
                     [--show-matching-path] [--filter.flags FLAGS]
                     [--filter.profile_name PROFILE_NAME]
                     [--filter.profile_attach PROFILE_ATTACH]
                     [--filter.profile_path PROFILE_PATH]

Check which profiles are used
'''  # noqa: E128

        expected_output_2 = \
'''
  -h, --help            show this help message and exit
  -s, --show-type {all,used,unused}
                        Type of profiles to show
  -j, --json            Output in JSON
  -d, --dir DIR         Path to profiles
  --show-matching-path  Show the path of a file matching the profile

Filtering options:
  Filters are used to reduce the output of information to only those entries
  that will match the filter. Filters use Python's regular expression syntax.

  --filter.flags FLAGS  Filter by flags
  --filter.profile_name PROFILE_NAME
                        Filter by profile name
  --filter.profile_attach PROFILE_ATTACH
                        Filter by profile attachment
  --filter.profile_path PROFILE_PATH
                        Filter by profile path
'''  # noqa: E128

        if sys.version_info[:2] < (3, 13):
            # Python 3.13 tweaked argparse output [1]. When running on older
            # Python versions, we adapt the expected output to match.
            #
            # https://github.com/python/cpython/pull/103372
            patches = [(
                '-s, --show-type {all,used,unused}',
                '-s {all,used,unused}, --show-type {all,used,unused}',
            ), (
                '-d, --dir DIR         Path to profiles',
                '-d DIR, --dir DIR     Path to profiles'
            )]
            for patch in patches:
                expected_output_2 = expected_output_2.replace(patch[0], patch[1])

        return_code, output = cmd(aashowusage_bin + ['--help'])
        result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
        self.assertEqual(expected_return_code, return_code, result + output)

        self.assertIn(expected_output_1, output)
        self.assertIn(expected_output_2, output)

    def test_show_unconfined_profiles(self):
        expected_return_code = 0
        return_code, output = cmd(aashowusage_bin + ['--filter.flags=unconfined', '-d', aa.profile_dir])
        result = 'Got return code {}, expected {}\n'.format(return_code, expected_return_code)
        self.assertEqual(expected_return_code, return_code, result + output)

        nb_profile = 0

        for line in output.splitlines():
            if line.startswith('  Profile '):
                nb_profile += 1

        command = ['grep', '-Er', r'flags=.*unconfined.*\{']

        # Remove disabled profiles from grep
        disable_dir = os.path.join(aa.profile_dir, 'disable')
        if os.path.isdir(disable_dir):
            for name in os.listdir(disable_dir):
                command.append('--exclude=' + name)

        command.extend(['--', aa.profile_dir])

        result = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.DEVNULL, text=True, check=False)
        self.assertEqual(
            len(result.stdout.splitlines()), nb_profile,
            "Error found {} profiles, expected {}\n\n Output was: \n {}. Grepped profiles are: {}".format(
                nb_profile, len(result.stdout.splitlines()), output, result.stdout)
        )


setup_aa(aa)  # Wrapper for aa.init_aa()
setup_all_loops(__name__)

# The location of the aa-show-usage utility can be overridden by setting
# the APPARMOR_SHOW_USAGE or USE_SYSTEM environment variable;
# this is useful for running these tests in an installed environment
aashowusage_bin = ["../aa-show-usage"]

if __name__ == '__main__':
    if 'APPARMOR_SHOW_USAGE' in os.environ:
        aashowusage_bin = [os.environ['APPARMOR_SHOW_USAGE']]
    elif 'USE_SYSTEM' in os.environ:
        aashowusage_bin = ['aa-show-usage']
    else:
        aashowusage_bin = [sys.executable] + aashowusage_bin

    unittest.main(verbosity=1)



#!/usr/bin/python3
#
#   Copyright (c) 2025 Canonical, Ltd. (All rights reserved)
#
#   This program is free software; you can redistribute it and/or
#   modify it under the terms of version 2 of the GNU General Public
#   License published by the Free Software Foundation.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, contact Canonical Ltd.
#

from testlib import write_file

def gen_file(test, xres, op, lhs, rhs, add_else):
    global count

    content = ''
    content += '#\n'
    content += '#=DESCRIPTION {}\n'.format(test)
    content += '#=EXRESULT {}\n'.format(xres)
    content += '#\n'
    if lhs['def'] and lhs != rhs:
        content += '{} = {}\n'.format(lhs['varname'], lhs['def'])
    if rhs['def']:
        content += '{} = {}\n'.format(rhs['varname'], rhs['def'])
    content += '/usr/bin/foo {\n'
    content += '  if {} {} {} {{\n'.format(lhs['varname'], op, rhs['varname'])
    content += '    /bin/true rix,\n'
    content += '  }'
    if add_else:
        content += ' else {\n'
        content += '    mount,\n'
        content += '  }\n'
    else:
        content += '\n'

    content += '}\n'

    write_file('simple_tests/generated_conditional', '{}{}-{}.sd'.format(test, '-else' if add_else else '', count), content)

    count += 1

ops = {'==': 'equals', '!=': 'notequals', 'in': 'in', '>': 'greater', '>=': 'greaterequals', '<': 'lesser', '<=': 'lesserequals'}

test_vars = [
    {'varname': '@{VAR_EMPTY}', 'def': '""', 'desc': 'empty', 'number': False},  # empty var
    {'varname': '@{VAR_ONE_STRING}', 'def': '/path/foo/', 'desc': 'var_one_string', 'number': False},  # one string in var
    {'varname': '@{VAR_ONE_NUMBER}', 'def': '10', 'desc': 'var_one_number', 'number': True},  # one number in var
    {'varname': '@{VAR_MULT_STRING}', 'def': '/path/foo/ /path/bar/', 'desc': 'var_mult_string', 'number': False},  # multiple strings in var
    {'varname': '@{VAR_MULT_NUMBER}', 'def': '10 2 3.1', 'desc': 'var_mult_number', 'number': False},  # multiple numbers in var
    {'varname': '@{VAR_MIXED}', 'def': '3 /foo 1 /bar/ 10 /path/foo/', 'desc': 'var_mixed', 'number': False},  # mixed var contents
    {'varname': '10', 'def': '', 'desc': 'number1', 'number': True},  # number directly
    {'varname': '9', 'def': '', 'desc': 'number2', 'number': True},  # number directly
    {'varname': '/path/foo/', 'def': '', 'desc': 'string1', 'number': False},  # string directly
    {'varname': '/path/baz/', 'def': '', 'desc': 'string2', 'number': False},  # string directly
]

def gen_files():
    for op in ops:
        for lhs in test_vars:
            for rhs in test_vars:
                for add_else in [True, False]:
                    test_description = lhs['desc'] + '-' + ops[op]  + '-' + rhs['desc']
                    xres = 'PASS' if lhs['number'] == rhs['number'] or op in ['in', '==', '!='] else 'FAIL'
                    gen_file(test_description, xres, op, lhs, rhs, add_else)

count = 0
gen_files()
print('Generated {} conditional tests'.format(count))



#!/usr/bin/env python3
import sys
import subprocess
import re
import os


VERBOSE = bool(os.environ.get('VERBOSE'))

NAME_RE     = re.compile(r'^Name:\s*(\S+)')
PERMS_ALL   = re.compile(r'^Perms:.*r.*:.*:.*\(/\{?,?\*\*,?}?\)')
ATTACH_RE   = re.compile(r'^Attachment:\s*(.+)')

RED = '\033[0;31m'
GREEN = '\033[0;32m'
NORMAL = '\033[0m'

def die(msg):
    print(RED + msg + NORMAL, file=sys.stderr)
    sys.exit(1)

def check_profile(name, prof_file, skip, attachment, lines):
    if skip:
        if VERBOSE:
            print('Profile "{}" skipped: {}'.format(name, skip))
        return

    pat = re.compile(r'^Perms:.*r.*:.*:.*\({}\)'.format(re.escape(attachment)))
    for l in lines:
        if pat.match(l):
            if VERBOSE:
                print(GREEN + 'Profile {} ({}): OK "{}" found'.format(prof_file, name, attachment) + NORMAL)
            return

    die('Profile {} ({}): ERROR: no Perms rule for "{}".'.format(prof_file, name, attachment))

def process_profile(profile, extra_args):
    cmd = ['../parser/apparmor_parser'] + extra_args + ['-d', profile]
    proc = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, text=True)
    if proc.returncode != 0:
        die('ERROR: Failed to parse "{}": {}'.format(profile, proc.stdout.strip()))

    lines = proc.stdout.splitlines()
    curr_name = ''
    attachment = ''
    skip = None
    in_entries = False
    block = []

    for line in lines:
        m = NAME_RE.match(line)
        if m:
            if curr_name:
                # check previously found profile
                check_profile(curr_name, profile, skip, attachment, block)

            # remember newly found profile
            curr_name = m.group(1)
            attachment = ''
            skip = None
            in_entries = False
            block = []
            continue

        if PERMS_ALL.match(line):
            skip = 'All files available'
            continue

        m = ATTACH_RE.match(line)
        if m:
            attachment = m.group(1)
            if attachment == '<NULL>':
                skip = 'no attachment'
            continue

        if line.strip() == '--- Entries ---':
            in_entries = True
            continue

        if in_entries:
            block.append(line)

    if not block:
        skip = 'no rules'
    # Last profile
    check_profile(curr_name, profile, skip, attachment, block)

if __name__ == '__main__':
    if len(sys.argv) < 2:
        print('Usage: {0} <profile-file> [parser_extra_args]'.format(sys.argv[0]))
        sys.exit(1)
    process_profile(sys.argv[1], sys.argv[2:])
